using System;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;

using Microsoft.SharePoint;
using Microsoft.SharePoint.WebControls;
using Microsoft.SharePoint.WebPartPages;
using Microsoft.SharePoint.Navigation;
using Microsoft.SharePoint.Administration;

namespace ThreeFive
{
    public class FeatureReceiver : SPFeatureReceiver
    {


        /// <summary>
        /// Executes when the feature is activated (i.e. the user clicks "activate")
        /// </summary>
        /// <param name="properties"></param>
        public override void FeatureActivated(SPFeatureReceiverProperties properties)
        {
            SPWebApplication app = (SPWebApplication)properties.Feature.Parent;
            foreach (ModificationEntry m in Entries)
            {
                app.WebConfigModifications.Add(
                  CreateChildNode(m)
                );
            }
            app.WebService.ApplyWebConfigModifications();
        }

        /// <summary>
        /// Executes when the feature is deactivated (i.e. the user clicks "deactivate")
        /// </summary>
        /// <param name="properties"></param>
        public override void FeatureDeactivating(SPFeatureReceiverProperties properties)
        {
            SPWebApplication app = (SPWebApplication)properties.Feature.Parent;
            foreach (ModificationEntry m in Entries)
            {
                app.WebConfigModifications.Remove(CreateChildNode(m));
            }
            app.WebService.ApplyWebConfigModifications();
        }

        /// <summary>
        /// NOT USED--Executes when the feature is installed
        /// </summary>
        /// <param name="properties"></param>
        public override void FeatureInstalled(SPFeatureReceiverProperties properties) { }

        /// <summary>
        /// NOT USED--Executes when the feature is uninstalling
        /// </summary>
        /// <param name="properties"></param>
        public override void FeatureUninstalling(SPFeatureReceiverProperties properties) { }

        /// <summary>
        /// Struct that represents the modification that is being made to the Web.Config
        /// </summary>
        private struct ModificationEntry
        {
            //members
            public string Name;
            public string XPath;
            public string Value;


            //constructor
            public ModificationEntry(string Name,
                                    string XPath,
                                    string Value)
            {
                this.Name = Name;
                this.XPath = XPath;
                this.Value = Value;

            }
        }

        /// <summary>
        /// Creates a child node in the Web.Config
        /// </summary>
        /// <param name="modEntry">Modification that is being made (TypeOf ModificationEntry Struct</param>
        /// <returns></returns>
        private SPWebConfigModification CreateChildNode(ModificationEntry modEntry)
        {
            // create and return SPWebConfigModification object
            SPWebConfigModification modification;
            modification = new SPWebConfigModification(modEntry.Name, modEntry.XPath);
            modification.Owner = "Unleashed.ThreeFive";
            modification.Sequence = 0;
            modification.Type = SPWebConfigModification.SPWebConfigModificationType.EnsureChildNode;
            modification.Value = modEntry.Value;
            return modification;
        }

        /// <summary>
        /// Array that contains the modifications to Web.Config.
        /// This will be used to both add and remove 
        /// modifications when the feature is activated and deactivated
        /// </summary>
        private ModificationEntry[] Entries = {
            new ModificationEntry( 
                "sectionGroup[@name='system.web.extensions']",
                "configuration/configSections",
                @"<sectionGroup name=""system.web.extensions"" type=""System.Web.Configuration.SystemWebExtensionsSectionGroup, System.Web.Extensions, Version=3.5.0.0, Culture=neutral, PublicKeyToken=31BF3856AD364E35""></sectionGroup>"),

            new ModificationEntry( 
                "sectionGroup[@name='scripting']",
                "configuration/configSections/sectionGroup[@name='system.web.extensions']",
                @"<sectionGroup name=""scripting"" type=""System.Web.Configuration.ScriptingSectionGroup, System.Web.Extensions, Version=3.5.0.0, Culture=neutral, PublicKeyToken=31BF3856AD364E35""></sectionGroup>"),


            new ModificationEntry( 
                "section[@name='scriptResourceHandler']",
                "configuration/configSections/sectionGroup[@name='system.web.extensions']/sectionGroup[@name='scripting']",
                @"<section name=""scriptResourceHandler"" type=""System.Web.Configuration.ScriptingScriptResourceHandlerSection, System.Web.Extensions, Version=3.5.0.0, Culture=neutral, PublicKeyToken=31BF3856AD364E35"" requirePermission=""false"" allowDefinition=""MachineToApplication""/>"),

            new ModificationEntry( 
                "sectionGroup[@name='webServices']",
                "configuration/configSections/sectionGroup[@name='system.web.extensions']/sectionGroup[@name='scripting']",
                @"<sectionGroup name=""webServices"" type=""System.Web.Configuration.ScriptingWebServicesSectionGroup, System.Web.Extensions, Version=3.5.0.0, Culture=neutral, PublicKeyToken=31BF3856AD364E35""></sectionGroup>"),

            new ModificationEntry( 
                "section[@name='jsonSerialization']",
                "configuration/configSections/sectionGroup[@name='system.web.extensions']/sectionGroup[@name='scripting']/sectionGroup[@name='webServices']",
                @"<section name=""jsonSerialization"" type=""System.Web.Configuration.ScriptingJsonSerializationSection, System.Web.Extensions, Version=3.5.0.0, Culture=neutral, PublicKeyToken=31BF3856AD364E35"" requirePermission=""false"" allowDefinition=""Everywhere"" />" ),

            new ModificationEntry( 
                "section[@name='profileService']",
                "configuration/configSections/sectionGroup[@name='system.web.extensions']/sectionGroup[@name='scripting']/sectionGroup[@name='webServices']",
                @"<section name=""profileService"" type=""System.Web.Configuration.ScriptingProfileServiceSection, System.Web.Extensions, Version=3.5.0.0, Culture=neutral, PublicKeyToken=31BF3856AD364E35"" requirePermission=""false"" allowDefinition=""MachineToApplication"" />"),

            new ModificationEntry( 
                "section[@name='authenticationService']",
                "configuration/configSections/sectionGroup[@name='system.web.extensions']/sectionGroup[@name='scripting']/sectionGroup[@name='webServices']",
                @"<section name=""authenticationService"" type=""System.Web.Configuration.ScriptingAuthenticationServiceSection, System.Web.Extensions, Version=3.5.0.0, Culture=neutral, PublicKeyToken=31BF3856AD364E35"" requirePermission=""false"" allowDefinition=""MachineToApplication"" />"),

            new ModificationEntry( 
               "section[@name='roleService']",
                "configuration/configSections/sectionGroup[@name='system.web.extensions']/sectionGroup[@name='scripting']/sectionGroup[@name='webServices']",
                @"<section name=""roleService"" type=""System.Web.Configuration.ScriptingRoleServiceSection, System.Web.Extensions, Version=3.5.0.0, Culture=neutral, PublicKeyToken=31BF3856AD364E35"" requirePermission=""false"" allowDefinition=""MachineToApplication"" />"),

            new ModificationEntry( 
                "pages",
            "configuration/system.web",
            @"<pages />"),

            new ModificationEntry( 
                "controls",
            "configuration/system.web/pages",
            @"<controls />"),

            new ModificationEntry( 
                 "add[@namespace='System.Web.UI']",
            "configuration/system.web/pages/controls",
            @"<add tagPrefix=""asp"" namespace=""System.Web.UI"" assembly=""System.Web.Extensions, Version=3.5.0.0, Culture=neutral, PublicKeyToken=31BF3856AD364E35""/>"),

            new ModificationEntry( 
                "add[@namespace='System.Web.UI.WebControls']",
            "configuration/system.web/pages/controls",
            @"<add tagPrefix=""asp"" namespace=""System.Web.UI.WebControls"" assembly=""System.Web.Extensions, Version=3.5.0.0, Culture=neutral, PublicKeyToken=31BF3856AD364E35""/>"),

            new ModificationEntry( 
                 "add[@assembly='System.Core, Version=3.5.0.0, Culture=neutral, PublicKeyToken=B77A5C561934E089']",
            "configuration/system.web/compilation/assemblies",
            @"<add assembly=""System.Core, Version=3.5.0.0, Culture=neutral, PublicKeyToken=B77A5C561934E089""/>"),

            new ModificationEntry( 
                "add[@assembly='System.Web.Extensions, Version=3.5.0.0, Culture=neutral, PublicKeyToken=31BF3856AD364E35']",
            "configuration/system.web/compilation/assemblies",
            @"<add assembly=""System.Web.Extensions, Version=3.5.0.0, Culture=neutral, PublicKeyToken=31BF3856AD364E35""/> "),

            new ModificationEntry( 
                "add[@assembly='System.Data.DataSetExtensions, Version=3.5.0.0, Culture=neutral, PublicKeyToken=B77A5C561934E089']",
            "configuration/system.web/compilation/assemblies",
            @"<add assembly=""System.Data.DataSetExtensions, Version=3.5.0.0, Culture=neutral, PublicKeyToken=B77A5C561934E089""/>"),

            new ModificationEntry( 
                "add[@assembly='System.Xml.Linq, Version=3.5.0.0, Culture=neutral, PublicKeyToken=B77A5C561934E089']",
            "configuration/system.web/compilation/assemblies",
            @"<add assembly=""System.Xml.Linq, Version=3.5.0.0, Culture=neutral, PublicKeyToken=B77A5C561934E089""/>"),

            new ModificationEntry( 
                "remove[@path='*.asmx']",
            "configuration/system.web/httpHandlers",
            @"<remove verb=""*"" path=""*.asmx""/> "),

            new ModificationEntry( 
               "add[@path='*.asmx']",
            "configuration/system.web/httpHandlers",
            @"<add verb=""*"" path=""*.asmx"" validate=""false"" type=""System.Web.Script.Services.ScriptHandlerFactory, System.Web.Extensions, Version=3.5.0.0, Culture=neutral, PublicKeyToken=31BF3856AD364E35""/>"),

            new ModificationEntry( 
                "add[@path='*_AppService.axd']",
            "configuration/system.web/httpHandlers",
            @"<add verb=""*"" path=""*_AppService.axd"" validate=""false"" type=""System.Web.Script.Services.ScriptHandlerFactory, System.Web.Extensions, Version=3.5.0.0, Culture=neutral, PublicKeyToken=31BF3856AD364E35""/>"),

            new ModificationEntry( 
                "add[@path='ScriptResource.axd']",
            "configuration/system.web/httpHandlers",
            @"<add verb=""GET,HEAD"" path=""ScriptResource.axd"" type=""System.Web.Handlers.ScriptResourceHandler, System.Web.Extensions, Version=3.5.0.0, Culture=neutral, PublicKeyToken=31BF3856AD364E35"" validate=""false""/>"),

            new ModificationEntry( 
                "add[@name='ScriptModule']",
            "configuration/system.web/httpModules",
            @"<add name=""ScriptModule"" type=""System.Web.Handlers.ScriptModule, System.Web.Extensions, Version=3.5.0.0, Culture=neutral, PublicKeyToken=31BF3856AD364E35""/>"),

            new ModificationEntry( 
                "SafeControl[@Assembly='System.Web.Extensions, Version=3.5.0.0, Culture=neutral, PublicKeyToken=31bf3856ad364e35'][@Namespace='System.Web.UI'][@TypeName='*']",
            "configuration/SharePoint/SafeControls",
            @"<SafeControl Assembly=""System.Web.Extensions, Version=3.5.0.0, Culture=neutral, PublicKeyToken=31bf3856ad364e35"" Namespace=""System.Web.UI"" TypeName=""*"" Safe=""True"" />"),

            new ModificationEntry( 
                "system.web.extensions",
            "configuration",
            @"<system.web.extensions />"),

            new ModificationEntry( 
               "scripting",
            "configuration/system.web.extensions",
            @"<scripting />"),

            new ModificationEntry( 
                "webServices",
            "configuration/system.web.extensions/scripting",
            @"<webServices />"),

            new ModificationEntry( 
                "system.webServer",
           "configuration",
           @"<system.webServer />"),

            new ModificationEntry( 
                "modules",
           "configuration/system.webServer",
           @"<modules />"),

            new ModificationEntry( 
               "handlers",
           "configuration/system.webServer",
           @"<handlers />"),

            new ModificationEntry( 
                "validation[@validateIntegratedModeConfiguration='false']",
                "configuration/system.webServer",
                @"<validation validateIntegratedModeConfiguration=""false""/>"),

            new ModificationEntry( 
                "remove[@name='ScriptModule']",
                "configuration/system.webServer/modules",
                @"<remove name=""ScriptModule"" />"),

            new ModificationEntry( 
                "add[@name='ScriptModule']",
                "configuration/system.webServer/modules",
                @"<add name=""ScriptModule"" preCondition=""managedHandler"" type=""System.Web.Handlers.ScriptModule, System.Web.Extensions, Version=3.5.0.0, Culture=neutral, PublicKeyToken=31BF3856AD364E35""/>"),

            new ModificationEntry( 
                "remove[@name='WebServiceHandlerFactory-Integrated']",
                "configuration/system.webServer/handlers",
                @"<remove name=""WebServiceHandlerFactory-Integrated"" />"),

            new ModificationEntry( 
                "remove[@name='ScriptHandlerFactory']",
                "configuration/system.webServer/handlers",
                @"<remove name=""ScriptHandlerFactory"" />"),

            new ModificationEntry( 
               "remove[@name='ScriptHandlerFactoryAppServices']",
                "configuration/system.webServer/handlers",
                @"<remove name=""ScriptHandlerFactoryAppServices"" />"),

            new ModificationEntry( 
                "remove[@name='ScriptResource']",
                "configuration/system.webServer/handlers",
                @"<remove name=""ScriptResource"" />"),

            new ModificationEntry( 
                "add[@name='ScriptHandlerFactory']",
                "configuration/system.webServer/handlers",
                @"<add name=""ScriptHandlerFactory"" verb=""*"" path=""*.asmx"" preCondition=""integratedMode"" type=""System.Web.Script.Services.ScriptHandlerFactory, System.Web.Extensions, Version=3.5.0.0, Culture=neutral, PublicKeyToken=31BF3856AD364E35""/>"),

            new ModificationEntry( 
               "add[@name='ScriptHandlerFactoryAppServices']",
               "configuration/system.webServer/handlers",
               @"<add name=""ScriptHandlerFactoryAppServices"" verb=""*"" path=""*_AppService.axd"" preCondition=""integratedMode"" type=""System.Web.Script.Services.ScriptHandlerFactory, System.Web.Extensions, Version=3.5.0.0, Culture=neutral, PublicKeyToken=31BF3856AD364E35""/>"),

            new ModificationEntry( 
               "add[@name='ScriptResource']",
                "configuration/system.webServer/handlers",
                @"<add name=""ScriptResource"" preCondition=""integratedMode"" verb=""GET,HEAD"" path=""ScriptResource.axd"" type=""System.Web.Handlers.ScriptResourceHandler, System.Web.Extensions, Version=3.5.0.0, Culture=neutral, PublicKeyToken=31BF3856AD364E35"" />"),

        };
    }
}
